@@ -11,13 +11,9 @@ from StringEncoding import EncodedString
 
 from ParseTreeTransforms import SkipDeclarations
 
-#def unwrap_node(node):
-#    while isinstance(node, ExprNodes.PersistentNode):
-#        node = node.arg
-#    return node
-
-# Temporary hack while PersistentNode is out of order
 def unwrap_node(node):
+    while isinstance(node, UtilNodes.ResultRefNode):
+        node = node.expression
     return node
 
 def is_common_value(a, b):
@@ -144,7 +140,6 @@ class IterationTransform(Visitor.VisitorTransform):
             relation2=relation2, bound2=bound2,
             step=step, body=node.body,
             else_clause=node.else_clause,
-            loopvar_node=node.target,
             from_range=True)
         return for_node
 
@@ -302,13 +297,17 @@ class SwitchTransform(Visitor.VisitorTransform):
     is common among all clauses and both var and value are ints. 
     """
     def extract_conditions(self, cond):
-    
-        if isinstance(cond, ExprNodes.CoerceToTempNode):
-            cond = cond.arg
+        while True:
+            if isinstance(cond, ExprNodes.CoerceToTempNode):
+                cond = cond.arg
+            elif isinstance(cond, UtilNodes.EvalWithTempExprNode):
+                # this is what we get from the FlattenInListTransform
+                cond = cond.subexpression
+            elif isinstance(cond, ExprNodes.TypecastNode):
+                cond = cond.operand
+            else:
+                break
 
-        if isinstance(cond, ExprNodes.TypecastNode):
-            cond = cond.operand
-    
         if (isinstance(cond, ExprNodes.PrimaryCmpNode) 
                 and cond.cascade is None 
                 and cond.operator == '=='
@@ -640,11 +639,9 @@ class FinalOptimizePhase(Visitor.CythonTransform):
             if node.function.name == 'isinstance':
                 type_arg = node.args[1]
                 if type_arg.type.is_builtin_type and type_arg.type.name == 'type':
-                    object_module = self.context.find_module('python_object')
-                    node.function.entry = object_module.lookup('PyObject_TypeCheck')
-                    if node.function.entry is None:
-                        return node # only happens when there was an error earlier
+                    from CythonScope import utility_scope
+                    node.function.entry = utility_scope.lookup('PyObject_TypeCheck')
                     node.function.type = node.function.entry.type
-                    PyTypeObjectPtr = PyrexTypes.CPtrType(object_module.lookup('PyTypeObject').type)
+                    PyTypeObjectPtr = PyrexTypes.CPtrType(utility_scope.lookup('PyTypeObject').type)
                     node.args[1] = ExprNodes.CastNode(node.args[1], PyTypeObjectPtr)
         return node
